import ctypes
from typing import List, Optional
from .__library__ import lib, get_error
from .camera_config import *
from .calibration import *
from .camera import *
from .data import *
from .csf_reader import *
from .gige_interface import *
from .usb_interface import *


class CsfWriter():
    """
    The csf writer class takes care of writing depth frames to disk.  The class has
    to be created from either a Camera or csf reader, because it needs information
    about the camera that was used to record the depth frames.  Therefore you should
    only write depth frames coming from one camera into the same writer

    """
    def __del__(self):
        if self._ptr:
            lib.tof_csf_writer_delete(self._ptr)
            self._ptr = ctypes.c_void_p()


    def __init__(self, file_location: Optional[str] = None, camera_config: Optional[CameraConfig] = None, calibration: Optional[Calibration] = None, serial: Optional[str] = None, camera: Optional[Camera] = None, ptr: Optional[ctypes.c_void_p] = None):
        """
        Construct new csf_writer class
        ----------

        Construct csf writer from calibration and configuration

        Parameters
        ----------
        file_location : str
                Path to where the csf file should be created
        camera_config : CameraConfig
                Camera configuration
        calibration : Calibration
                Calibration
        serial : str
                Serial number of camera
        ----------

        Construct the csf writer from a camera

        Parameters
        ----------
        file_location : str
                Path to where the csf file should be created
        camera : Camera
                The camera to get the camera config and calibration file from
        """
        self._ptr = ctypes.c_void_p()

        if ptr != None:
            self._ptr = ptr
        elif file_location != None and camera_config != None and calibration != None and serial != None:
            self._ptr = lib.tof_csf_writer_new(file_location.encode("utf-8"), camera_config._ptr, calibration._ptr, serial.encode("utf-8"), get_error())
        elif file_location != None and camera != None:
            self._ptr = lib.tof_csf_writer_new_from_camera(file_location.encode("utf-8"), camera._ptr, get_error())
        else:
            raise RuntimeError("Can't construct empty class")

    def writeFrame(self, frame: Data) -> None:
        """
        Write frame to csf file

        Parameters
        ----------
        frame : Data
            A data frame
        """
        return lib.tof_csf_writer_write_frame(self._ptr, frame._ptr, get_error())


def createCsfWriterReader(file_location: str, reader: CsfReader) -> CsfWriter:
    """
    Create csf writer from csf reader

    Parameters
    ----------
    file_location : str
        Path to where the csf file should be created
    reader : CsfReader
        A csf reader

    Returns
    ----------
    CsfWriter
        csf writer
    """
    return CsfWriter(ptr = lib.tof_create_csf_writer_reader(file_location.encode("utf-8"), reader._ptr, get_error()))


def createCsfWriterCamera(file_location: str, camera: Camera) -> CsfWriter:
    """
    Create csf writer from camera

    Parameters
    ----------
    file_location : str
        Path to where the csf file should be created
    camera : Camera
        A camera

    Returns
    ----------
    CsfWriter
        csf writer
    """
    return CsfWriter(ptr = lib.tof_create_csf_writer_camera(file_location.encode("utf-8"), camera._ptr, get_error()))


def createCsfWriterGigeInterface(file_location: str, gige_interface: GigeInterface) -> CsfWriter:
    """
    Create csf writer from the GigE interface

    Parameters
    ----------
    file_location : str
        Path to where the csf file should be created
    gige_interface : GigeInterface
        A GigE interface

    Returns
    ----------
    CsfWriter
        csf writer
    """
    return CsfWriter(ptr = lib.tof_create_csf_writer_gige_interface(file_location.encode("utf-8"), gige_interface._ptr, get_error()))


def createCsfWriterUsbInterface(file_location: str, usb_interface: UsbInterface) -> CsfWriter:
    """
    Create csf writer from the USB interface

    Parameters
    ----------
    file_location : str
        Path to where the csf file should be created
    usb_interface : UsbInterface
        A USB interface

    Returns
    ----------
    CsfWriter
        csf writer
    """
    return CsfWriter(ptr = lib.tof_create_csf_writer_usb_interface(file_location.encode("utf-8"), usb_interface._ptr, get_error()))


